@@ -2,10 +2,10 @@ from __future__ import annotations
 import os
 
 import asyncio
-
+from functools import partial
 import logging
 import signal
-from typing import Any, ClassVar, Type, TypeVar
+from typing import Any, Callable, ClassVar, Type, TypeVar
 import warnings
 
 from rich.control import Control
@@ -13,20 +13,24 @@ import rich.repr
 from rich.screen import Screen
 from rich import get_console
 from rich.console import Console, RenderableType
+from rich.style import Style
 from rich.traceback import Traceback
 
 from . import events
 from . import actions
 from ._animator import Animator
-from .geometry import Region
+from .binding import Bindings, NoBinding
+from .geometry import Point, Region
 from ._context import active_app
+from ._event_broker import extract_handler_actions, NoHandler
 from .keys import Binding
 from .driver import Driver
-from .layouts.dock import DockLayout, Dock, DockEdge, DockOptions
+from .layouts.dock import DockLayout, Dock
 from ._linux_driver import LinuxDriver
 from .message_pump import MessagePump
 from .message import Message
-from .view import DockView, View
+from .view import View
+from .views import DockView
 from .widget import Widget, Widget, Reactive
 
 log = logging.getLogger("rich")
@@ -70,15 +74,14 @@ class App(MessagePump):
         console: Console = None,
         screen: bool = True,
         driver_class: Type[Driver] = None,
-        title: str = "Megasoma Application",
+        title: str = "Textual Application",
     ):
-        super().__init__()
         self.console = console or get_console()
         self._screen = screen
         self.driver_class = driver_class or LinuxDriver
-        self.title = title
+        self._title = title
         self._layout = DockLayout()
-        self._view_stack: list[View] = [View()]
+        self._view_stack: list[View] = []
         self.children: set[MessagePump] = set()
 
         self.focused: Widget | None = None
@@ -86,11 +89,20 @@ class App(MessagePump):
         self.mouse_captured: Widget | None = None
         self._driver: Driver | None = None
 
-        self._bindings: dict[str, Binding] = {}
         self._docks: list[Dock] = []
         self._action_targets = {"app", "view"}
         self._animator = Animator(self)
         self.animate = self._animator.bind(self)
+        self.mouse_position = Point(0, 0)
+        self.bindings = Bindings()
+        self._title = title
+
+        self.bindings.bind("ctrl+c", "quit")
+
+        super().__init__()
+
+    title: Reactive[str] = Reactive("Textual")
+    sub_title: Reactive[str] = Reactive("")
 
     def __rich_repr__(self) -> rich.repr.RichReprResult:
         yield "title", self.title
@@ -106,18 +118,18 @@ class App(MessagePump):
     def view(self) -> View:
         return self._view_stack[-1]
 
-    @property
-    def bindings(self) -> dict[str, Binding]:
-        return self._bindings
-
-    async def bind(self, keys: str, action: str, description: str = "") -> None:
-        all_keys = [key.strip() for key in keys.split(",")]
-        for key in all_keys:
-            self._bindings[key] = Binding(action, description)
+    async def bind(
+        self, keys: str, action: str, description: str = "", show: bool = False
+    ) -> None:
+        self.bindings.bind(keys, action, description, show=show)
 
     @classmethod
     def run(
-        cls, console: Console = None, screen: bool = True, driver: Type[Driver] = None
+        cls,
+        console: Console = None,
+        screen: bool = True,
+        driver: Type[Driver] = None,
+        **kwargs,
     ):
         """Run the app.
 
@@ -128,8 +140,7 @@ class App(MessagePump):
         """
 
         async def run_app() -> None:
-            app = cls(console=console, screen=screen, driver_class=driver)
-
+            app = cls(console=console, screen=screen, driver_class=driver, **kwargs)
             await app.process_messages()
 
         asyncio.run(run_app())
@@ -137,7 +148,7 @@ class App(MessagePump):
     async def push_view(self, view: ViewType) -> ViewType:
         await self.register(view)
         view.set_parent(self)
-        self._view_stack[0] = view
+        self._view_stack.append(view)
         return view
 
     def on_keyboard_interupt(self) -> None:
@@ -181,7 +192,15 @@ class App(MessagePump):
 
     async def capture_mouse(self, widget: Widget | None) -> None:
         """Send all Mouse events to a given widget."""
+        if widget == self.mouse_captured:
+            return
+        if self.mouse_captured is not None:
+            await self.mouse_captured.post_message(
+                events.MouseReleased(self, self.mouse_position)
+            )
         self.mouse_captured = widget
+        if widget is not None:
+            await widget.post_message(events.MouseCaptured(self, self.mouse_position))
 
     async def process_messages(self) -> None:
         log.debug("driver=%r", self.driver_class)
@@ -189,13 +208,13 @@ class App(MessagePump):
         # loop.add_signal_handler(signal.SIGINT, self.on_keyboard_interupt)
         driver = self._driver = self.driver_class(self.console, self)
         active_app.set(self)
+
+        await self.push_view(View())
+
         self.view.set_parent(self)
         await self.register(self.view)
 
-        if hasattr(self, "on_load"):
-            await self.on_load(events.Load(sender=self))
-
-        await self.post_message(events.Startup(sender=self))
+        await self.dispatch_message(events.Load(sender=self))
 
         try:
             driver.start_application_mode()
@@ -204,17 +223,30 @@ class App(MessagePump):
             log.exception("error starting application mode")
         else:
             traceback: Traceback | None = None
-            await self.animator.start()
+
             try:
+                self.title = self._title
+                await self.post_message(events.Startup(sender=self))
+                self.require_layout()
+                await self.animator.start()
+
                 await super().process_messages()
+                await self.animator.stop()
+
+                while self.children:
+                    child = self.children.pop()
+                    log.debug("closing %r", child)
+                    await child.close_messages()
+
+                while self._view_stack:
+                    view = self._view_stack.pop()
+                    await view.close_messages()
             except Exception:
                 traceback = Traceback(show_locals=True)
-
-            await self.animator.stop()
-            await self.view.close_messages()
-            driver.stop_application_mode()
-            if traceback is not None:
-                self.console.print(traceback)
+            finally:
+                driver.stop_application_mode()
+                if traceback is not None:
+                    self.console.print(traceback)
 
     def require_repaint(self) -> None:
         self.refresh()
@@ -222,6 +254,12 @@ class App(MessagePump):
     def require_layout(self) -> None:
         self.view.require_layout()
 
+    async def call_later(self, callback: Callable, *args, **kwargs) -> None:
+        await self.post_message(events.Idle(self))
+        await self.post_message(
+            events.Callback(self, partial(callback, *args, **kwargs))
+        )
+
     async def message_update(self, message: Message) -> None:
         self.refresh()
 
@@ -235,6 +273,7 @@ class App(MessagePump):
 
     async def shutdown(self):
         driver = self._driver
+        assert driver is not None
         driver.disable_input()
         await self.close_messages()
 
@@ -266,12 +305,17 @@ class App(MessagePump):
 
     async def on_event(self, event: events.Event) -> None:
         if isinstance(event, events.Key):
-            binding = self._bindings.get(event.key, None)
-            if binding is not None:
+            try:
+                binding = self.bindings.get_key(event.key)
+            except NoBinding:
+                pass
+            else:
                 await self.action(binding.action)
                 return
 
         if isinstance(event, events.InputEvent):
+            if isinstance(event, events.MouseEvent):
+                self.mouse_position = Point(event.x, event.y)
             if isinstance(event, events.Key) and self.focused is not None:
                 await self.focused.forward_event(event)
             await self.view.forward_event(event)
@@ -279,7 +323,10 @@ class App(MessagePump):
             await super().on_event(event)
 
     async def action(
-        self, action: str, default_namespace: object | None = None
+        self,
+        action: str,
+        default_namespace: object | None = None,
+        modifiers: set[str] | None = None,
     ) -> None:
         """Perform an action.
 
@@ -302,13 +349,27 @@ class App(MessagePump):
     async def dispatch_action(
         self, namespace: object, action_name: str, params: Any
     ) -> None:
+        _rich_traceback_guard = True
         method_name = f"action_{action_name}"
         method = getattr(namespace, method_name, None)
         if method is not None:
             await method(*params)
 
-    async def on_load(self, event: events.Load) -> None:
-        pass
+    async def broker_event(
+        self, event_name: str, event: events.Event, default_namespace: object | None
+    ) -> bool:
+        try:
+            style = getattr(event, "style")
+        except AttributeError:
+            return False
+        try:
+            modifiers, action = extract_handler_actions(event_name, style.meta)
+        except NoHandler:
+            return False
+        await self.action(
+            action, default_namespace=default_namespace, modifiers=modifiers
+        )
+        return True
 
     async def on_shutdown_request(self, event: events.ShutdownRequest) -> None:
         log.debug("shutdown request")
@@ -317,21 +378,6 @@ class App(MessagePump):
     async def on_resize(self, event: events.Resize) -> None:
         await self.view.post_message(event)
 
-    async def on_mouse_move(self, event: events.MouseMove) -> None:
-        await self.view.post_message(event)
-
-    async def on_mouse_down(self, event: events.MouseDown) -> None:
-        await self.view.post_message(event)
-
-    async def on_mouse_up(self, event: events.MouseUp) -> None:
-        await self.view.post_message(event)
-
-    async def on_mouse_scroll_up(self, event: events.MouseScrollUp) -> None:
-        await self.view.post_message(event)
-
-    async def on_mouse_scroll_down(self, event: events.MouseScrollUp) -> None:
-        await self.view.post_message(event)
-
     async def action_quit(self) -> None:
         await self.shutdown()
 
@@ -387,7 +433,7 @@ if __name__ == "__main__":
 
         show_bar: Reactive[bool] = Reactive(False)
 
-        def watch_show_bar(self, show_bar: bool) -> None:
+        async def watch_show_bar(self, show_bar: bool) -> None:
             self.animator.animate(self.bar, "layout_offset_x", -40 if show_bar else 0)
 
         async def action_toggle_sidebar(self) -> None:
@@ -397,7 +443,7 @@ if __name__ == "__main__":
 
             view = await self.push_view(DockView())
 
-            header = Header(self.title)
+            header = Header()
             footer = Footer()
             self.bar = Placeholder(name="left")
             footer.add_key("b", "Toggle sidebar")
